import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import '../components/my_elevated_button.dart';
import '../components/text_bold.dart';
import '../constants.dart';
import '../enumerations.dart';
import '../forms/risks_form.dart';
import '../services/auth/auth_service.dart';
import '../utils/helper_dialogs.dart';

class CultureValuesFormPage extends StatefulWidget {
  const CultureValuesFormPage(
      {super.key, required this.isRegProcess, required this.isEditMode});

  final bool isRegProcess;
  final bool isEditMode;

  @override
  CultureValuesFormPageState createState() => CultureValuesFormPageState();
}

class CultureValuesFormPageState extends State<CultureValuesFormPage> {
  Map<WorkValueOption, bool> selectedValueOptions = {
    WorkValueOption.transparency: false,
    WorkValueOption.innovation: false,
    WorkValueOption.teamwork: false,
    WorkValueOption.workLifeBalance: false,
  };
  CultureOption? selectedCultureOption;

  final AuthService _authService = AuthService();

  final int minWorkValueOption = 1;
  final int maxWorkValueOption = 2;

  @override
  void initState() {
    super.initState();
    _loadDataFromFirebase();
  }

  Future<void> _loadDataFromFirebase() async {
    final userDoc = FirebaseFirestore.instance
        .collection(Constants.dbCollectionUsers)
        .doc(_authService.getCurrentUser()!.uid);
    final snapshot = await userDoc.get();

    if (snapshot.exists) {
      final data = snapshot.data();

      setState(() {
        // Load WorkValue options
        if (data != null) {
          if (data[Constants.dbFieldUsersWorkValues] != null) {
            for (var option in WorkValueOption.values) {
              selectedValueOptions[option] =
                  data[Constants.dbFieldUsersWorkValues]
                      .contains(option.toString());
            }
          }
          // Load Culture option
          if (data[Constants.dbFieldUsersCorpCulture] != null) {
            selectedCultureOption = CultureOption.values.firstWhereOrNull(
              (e) => e.toString() == data[Constants.dbFieldUsersCorpCulture],
            );
          }
        }
      });
    }
  }

  Future<bool> _saveDataToFirebase() async {
    try {
      final userDoc = FirebaseFirestore.instance
          .collection(Constants.dbCollectionUsers)
          .doc(_authService.getCurrentUser()!.uid);

      await userDoc.set(
        {
          Constants.dbFieldUsersWorkValues: selectedValueOptions.entries
              .where((entry) => entry.value)
              .map((entry) => entry.key.toString())
              .toList(),
          Constants.dbFieldUsersCorpCulture: selectedCultureOption?.toString(),
        },
        SetOptions(merge: true), // avoid overwriting existing data
      );
      return true;
    } catch (e) {
      _showSnackBar('Error saving data: ${e.toString()}');
      return false;
    }
  }

  void _showSnackBar(String message) {
    showErrorSnackBar(context, message);
  }

  bool isWorkValueSelected() {
    int selectedCount =
        selectedValueOptions.values.where((value) => value).length;
    return selectedCount >= minWorkValueOption &&
        selectedCount <= maxWorkValueOption;
  }

  Future<void> handleSubmit() async {
    if (!isWorkValueSelected()) {
      _showSnackBar(
          'Please select at least ${minWorkValueOption.toString()} and at most ${maxWorkValueOption.toString()} work life values.');
      return;
    }
    if (selectedCultureOption == null) {
      _showSnackBar('Please select a corporate culture.');
      return;
    }
    // Handle the form submission logic here
    bool success = await _saveDataToFirebase();
    if (success && mounted) {
      if (widget.isRegProcess) {
        Navigator.push(
          context,
          MaterialPageRoute(
            //
            // set following registration page HERE
            // this Page (CorporateCultureForm) is optional as of 22.06.24
            // and therefore not called anymore during RegProcess
            //
            builder: (context) => RisksFormPage(
              isRegProcess: widget.isRegProcess,
              isEditMode: false,
            ),
          ),
        );
      } else {
        if (widget.isEditMode == true) {
          // pass selectedOptions data back to caller
          Navigator.pop(context, {
            Constants.dbFieldUsersWorkValues: selectedValueOptions.entries
                .where((entry) => entry.value)
                .map((entry) => entry.key)
                .toList(),
            Constants.dbFieldUsersCorpCulture: selectedCultureOption,
          });
        } else {
          Navigator.pop(context);
        }
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Personal preferences'),
        actions: [
          if (widget.isEditMode && !widget.isRegProcess)
            IconButton(
              onPressed: handleSubmit,
              icon: const Icon(Icons.save),
            )
        ],
      ),
      body: Padding(
        padding: const EdgeInsets.all(16.0),
        child: SingleChildScrollView(
          child: Column(
            children: [
              const TextBold(text: 'Work life and corporate culture'),
              const Align(
                alignment: Alignment.centerLeft,
                child: Text('Which corporate culture suits you best?'),
              ),
              ...CultureOption.values.map((option) {
                return RadioListTile(
                  title: Text(option.displayName),
                  value: option,
                  groupValue: selectedCultureOption,
                  onChanged: (CultureOption? value) {
                    setState(() {
                      selectedCultureOption = value;
                    });
                  },
                );
              }),
              const SizedBox(height: 40),
              const Align(
                alignment: Alignment.centerLeft,
                child: Text(
                  'What values are most important to you in your working environment?',
                ),
              ),
              ...WorkValueOption.values.map((option) {
                return CheckboxListTile(
                  title: Text(option.displayName),
                  value: selectedValueOptions[option],
                  onChanged: (bool? value) {
                    setState(() {
                      int selectedCount = selectedValueOptions.values
                          .where((value) => value)
                          .length;
                      if (value == false ||
                          selectedCount < maxWorkValueOption) {
                        selectedValueOptions[option] = value!;
                      } else if (value == true &&
                          selectedCount >= maxWorkValueOption) {
                        _showSnackBar(
                          'You can only select a maximum of two values.',
                        );
                      }
                    });
                  },
                );
              }),
              const SizedBox(height: 20),
              MyElevatedButton(
                onPressed: handleSubmit,
                child: Text(widget.isRegProcess ? 'Save and continue' : 'Save'),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
