import 'package:flutter/material.dart';
import '../components/my_elevated_button.dart';
import '../components/text_bold.dart';
import '../utils/helper.dart';

class ProfileCategoryForm<T> extends StatefulWidget {
  final String title;
  final String header;
  final String description;
  final List<T> options; // T to make it work with our different Enums
  final int minSelections;
  final int maxSelections;
  final Function(List<T>) onSave;
  final List<T> preSelectedOptions;
  final String? saveButtonText;
  final bool hideSaveButton;

  const ProfileCategoryForm({
    super.key,
    required this.title,
    required this.header,
    required this.description,
    required this.options,
    required this.minSelections,
    required this.maxSelections,
    required this.onSave,
    required this.preSelectedOptions,
    this.saveButtonText,
    this.hideSaveButton = false,
  });

  @override
  ProfileCategoryFormState<T> createState() => ProfileCategoryFormState<T>();
}

class ProfileCategoryFormState<T> extends State<ProfileCategoryForm<T>> {
  late List<T> _selectedOptions = [];
  bool _isSnackBarVisible = false; // Track SnackBar visibility

  @override
  void initState() {
    super.initState();
    _selectedOptions = List.from(widget.preSelectedOptions);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.title),
        actions: [
          if (!widget.hideSaveButton)
            IconButton(
              onPressed: _saveButtonClicked,
              icon: const Icon(Icons.save),
            )
        ],
      ),
      body: SingleChildScrollView(
        padding: const EdgeInsets.all(16.0),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: [
            TextBold(text: widget.header),
            const SizedBox(height: 16),
            Text(widget.description),
            const SizedBox(height: 16),
            Wrap(
              spacing: 8.0,
              children: List.generate(widget.options.length, (index) {
                final option = widget.options[index];
                return ChoiceChip(
                  label: Text(getDisplayText(option)),
                  selected: _selectedOptions.contains(option),
                  onSelected: (selected) {
                    setState(() {
                      if (selected) {
                        if (_selectedOptions.length < widget.maxSelections) {
                          _selectedOptions.add(option);
                        } else {
                          _showSnackBar(
                            'Maximum selections reached for ${widget.title}',
                          );
                        }
                      } else {
                        _selectedOptions.remove(option);
                      }
                    });
                  },
                );
              }),
            ),
            const SizedBox(height: 16.0),
            if (!widget.hideSaveButton)
              MyElevatedButton(
                onPressed: _saveButtonClicked,
                child: Text(widget.saveButtonText ?? 'Save'),
              ),
          ],
        ),
      ),
    );
  }

  void _saveButtonClicked() {
    if (_selectedOptions.length >= widget.minSelections) {
      widget.onSave(_selectedOptions);
    } else {
      _showSnackBar('No selection made for ${widget.title}');
    }
  }

  void _showSnackBar(String message) {
    if (!_isSnackBarVisible) {
      _isSnackBarVisible = true;
      ScaffoldMessenger.of(context)
          .showSnackBar(
            SnackBar(
              content: Text(
                message,
                style: TextStyle(
                  fontSize: 16,
                  color: Theme.of(context).colorScheme.onErrorContainer,
                ),
              ),
              backgroundColor: Theme.of(context).colorScheme.errorContainer,
              //duration: Duration(seconds: 2),
              action: SnackBarAction(
                label: 'Dismiss',
                textColor: Theme.of(context).colorScheme.primary,
                onPressed: () {
                  ScaffoldMessenger.of(context).hideCurrentSnackBar();
                  _isSnackBarVisible = false;
                },
              ),
            ),
          )
          .closed
          .then((value) {
        _isSnackBarVisible = false;
      });
    }
  }
}
